<?php

interface ISessionHandler {
	public function OpenSession($SavePath, $SessionName);
	public function CloseSession();
	public function ReadFromSession($SessId);
	public function WriteToSession($SessId, $SessionData);
	public function DestroySession($SessId);
	public function CollectGarbage($MaxLifeTime);
};

class SecureSessionHandler implements ISessionHandler {
	
	private $save_path = NULL;
	private $conn = NULL;

	public function OpenSession($SavePath, $SessionName) {
		try {
			$this->conn = new PDO("pgsql:host=localhost port=5432 dbname=securedb user=postgres password=postgrespass");
			return TRUE;
		} catch ( Exception $error ) {
			return FALSE;
		}	
	}

	public function CloseSession() {
		$this->conn = NULL;
	}

	public function ReadFromSession($SessId) {
		$query = "SELECT session_data FROM secure_sessions WHERE session_id = :sessid";
		$st = $this->conn->prepare($query);
		$st->setFetchMode(PDO::FETCH_OBJ);
		$res = $st->execute(array(':sessid' => (string)$SessId));
		if ( $res ) {
			$row = $st->fetch();
			if ( $row ) {
				return $row->session_data;
			}	
		} 
		return '';
	}

	public function WriteToSession($SessId, $SessionData) {
		$query = "SELECT session_id FROM secure_sessions WHERE session_id = :sessid";
		$st = $this->conn->prepare($query);
		$res = $st->execute(array(':sessid' => (string)$SessId));
		$row = $st->fetch();
		if ( $row ) {
			$query = "UPDATE secure_sessions SET session_data = :sessdata WHERE session_id = :sessid";
			$exst = $this->conn->prepare($query);
		} else {
			$query = "INSERT INTO secure_sessions (session_id, session_data) VALUES(:sessid, :sessdata)";
			$exst = $this->conn->prepare($query);
		} 
		return $exst->execute(array(':sessid' => (string)$SessId, ':sessdata' => (string)$SessionData));
	}

	public function DestroySession($SessId) {
		$query = "DELETE FROM secure_sessions WHERE session_id = :sessid";
		$st = $this->conn->prepare($query);
		$res = $st->execute(array(':sessid' => (string)$SessId));
		return $res;
	}

	public function CollectGarbage($MaxLifeTime) {
		return 0;
	}
}
$session_handler = new SecureSessionHandler();
$open_handler = array($session_handler, 'OpenSession');
$close_handler = array($session_handler, 'CloseSession');
$read_handler = array($session_handler, 'ReadFromSession');
$write_handler = array($session_handler, 'WriteToSession');
$destroy_handler = array($session_handler, 'DestroySession');
$gc_handler = array($session_handler, 'CollectGarbage');

session_set_save_handler($open_handler, $close_handler, $read_handler, $write_handler, $destroy_handler, $gc_handler);
?>
